@@ -1,4 +1,4 @@
-//Copyright (c) 2017 Ultimaker B.V.
+//Copyright (c) 2018 Ultimaker B.V.
 //CuraEngine is released under the terms of the AGPLv3 or higher.
 
 #include "FffPolygonGenerator.h"
@@ -31,9 +31,11 @@
 #include "PrintFeature.h"
 #include "ConicalOverhang.h"
 #include "TopSurface.h"
+#include "TreeSupport.h"
 #include "progress/ProgressEstimator.h"
 #include "progress/ProgressStageEstimator.h"
 #include "progress/ProgressEstimatorLinear.h"
+#include "settings/AdaptiveLayerHeights.h"
 
 
 namespace cura
@@ -77,30 +79,71 @@ bool FffPolygonGenerator::sliceModel(MeshGroup* meshgroup, TimeKeeper& timeKeepe
     storage.model_size = storage.model_max - storage.model_min;
 
     log("Slicing model...\n");
+
+    // regular layers
+    int slice_layer_count = 0;
+    int layer_thickness = getSettingInMicrons("layer_height");
     int initial_layer_thickness = getSettingInMicrons("layer_height_0");
-    if(initial_layer_thickness <= 0) //Initial layer height of 0 is not allowed. Negative layer height is nonsense.
+
+    // Initial layer height of 0 is not allowed. Negative layer height is nonsense.
+    if (initial_layer_thickness <= 0)
     {
         logError("Initial layer height %i is disallowed.\n", initial_layer_thickness);
         return false;
     }
-    int layer_thickness = getSettingInMicrons("layer_height");
-    if(layer_thickness <= 0) //Layer height of 0 is not allowed. Negative layer height is nonsense.
+
+    // Layer height of 0 is not allowed. Negative layer height is nonsense.
+    if(layer_thickness <= 0)
     {
         logError("Layer height %i is disallowed.\n", layer_thickness);
         return false;
     }
-    int slice_layer_count = (storage.model_max.z - initial_layer_thickness) / layer_thickness + 2;
-    if (slice_layer_count <= 0) //Model is shallower than layer_height_0, so not even the first layer is sliced. Return an empty model then.
+
+    // variable layers
+    AdaptiveLayerHeights* adaptive_layer_heights = nullptr;
+    bool use_variable_layer_heights = getSettingBoolean("adaptive_layer_height_enabled");
+
+    if (use_variable_layer_heights)
     {
-        return true; //This is NOT an error state!
+        // Calculate adaptive layer heights
+        coord_t variable_layer_height_max_variation = getSettingInMicrons("adaptive_layer_height_variation");
+        coord_t variable_layer_height_variation_step = getSettingInMicrons("adaptive_layer_height_variation_step");
+        double adaptive_threshold = getSettingInAngleDegrees("adaptive_layer_height_threshold");
+        adaptive_layer_heights = new AdaptiveLayerHeights(meshgroup, layer_thickness, initial_layer_thickness,
+                                                          variable_layer_height_max_variation,
+                                                          variable_layer_height_variation_step, adaptive_threshold);
+
+        // Get the amount of layers
+        slice_layer_count = adaptive_layer_heights->getLayerCount();
+    }
+    else
+    {
+        slice_layer_count = (storage.model_max.z - initial_layer_thickness) / layer_thickness + 2;
+    }
+
+    // Model is shallower than layer_height_0, so not even the first layer is sliced. Return an empty model then.
+    if (slice_layer_count <= 0)
+    {
+        return true; // This is NOT an error state!
     }
 
     std::vector<Slicer*> slicerList;
     for(unsigned int mesh_idx = 0; mesh_idx < meshgroup->meshes.size(); mesh_idx++)
     {
+        // Check if adaptive layers is populated to prevent accessing a method on NULL
+        std::vector<AdaptiveLayer>* adaptive_layer_height_values = {};
+        if (adaptive_layer_heights != NULL) {
+            adaptive_layer_height_values = adaptive_layer_heights->getLayers();
+        }
+
         Mesh& mesh = meshgroup->meshes[mesh_idx];
-        Slicer* slicer = new Slicer(&mesh, initial_layer_thickness, layer_thickness, slice_layer_count, mesh.getSettingBoolean("meshfix_keep_open_polygons"), mesh.getSettingBoolean("meshfix_extensive_stitching"));
+        Slicer* slicer = new Slicer(&mesh, initial_layer_thickness, layer_thickness, slice_layer_count,
+                                    mesh.getSettingBoolean("meshfix_keep_open_polygons"),
+                                    mesh.getSettingBoolean("meshfix_extensive_stitching"),
+                                    use_variable_layer_heights, adaptive_layer_height_values);
+
         slicerList.push_back(slicer);
+
         /*
         for(SlicerLayer& layer : slicer->layers)
         {
@@ -109,11 +152,12 @@ bool FffPolygonGenerator::sliceModel(MeshGroup* meshgroup, TimeKeeper& timeKeepe
             sendPolygons("openoutline", layer_nr, layer.openPolygonList);
         }
         */
+
         Progress::messageProgress(Progress::Stage::SLICING, mesh_idx + 1, meshgroup->meshes.size());
     }
 
-    meshgroup->clear();///Clear the mesh face and vertex data, it is no longer needed after this point, and it saves a lot of memory.
-
+    // Clear the mesh face and vertex data, it is no longer needed after this point, and it saves a lot of memory.
+    meshgroup->clear();
 
     Mold::process(storage, slicerList, layer_thickness);
 
@@ -142,9 +186,9 @@ bool FffPolygonGenerator::sliceModel(MeshGroup* meshgroup, TimeKeeper& timeKeepe
     {
         Mesh& mesh = storage.meshgroup->meshes[meshIdx];
         Slicer* slicer = slicerList[meshIdx];
-        if (!mesh.getSettingBoolean("anti_overhang_mesh") && !mesh.getSettingBoolean("infill_mesh"))
+        if (!mesh.getSettingBoolean("anti_overhang_mesh") && !mesh.getSettingBoolean("infill_mesh") && !mesh.getSettingBoolean("cutting_mesh"))
         {
-            storage.print_layer_count = std::max(storage.print_layer_count, (unsigned int)slicer->layers.size());
+            storage.print_layer_count = std::max(storage.print_layer_count, slicer->layers.size());
         }
     }
     storage.support.supportLayers.resize(storage.print_layer_count);
@@ -159,21 +203,41 @@ bool FffPolygonGenerator::sliceModel(MeshGroup* meshgroup, TimeKeeper& timeKeepe
         storage.meshes.emplace_back(&meshgroup->meshes[meshIdx], slicer->layers.size()); // new mesh in storage had settings from the Mesh
         SliceMeshStorage& meshStorage = storage.meshes.back();
 
+        // only create layer parts for normal meshes
         const bool is_support_modifier = AreaSupport::handleSupportModifierMesh(storage, mesh, slicer);
-
         if (!is_support_modifier)
-        { // only create layer parts for normal meshes
+        {
             createLayerParts(meshStorage, slicer, mesh.getSettingBoolean("meshfix_union_all"), mesh.getSettingBoolean("meshfix_union_all_remove_holes"));
         }
 
+        // check one if raft offset is needed
         bool has_raft = getSettingAsPlatformAdhesion("adhesion_type") == EPlatformAdhesion::RAFT;
-        //Add the raft offset to each layer.
+
+        // calculate the height at which each layer is actually printed (printZ)
         for (unsigned int layer_nr = 0; layer_nr < meshStorage.layers.size(); layer_nr++)
         {
             SliceLayer& layer = meshStorage.layers[layer_nr];
-            meshStorage.layers[layer_nr].printZ =
-                getSettingInMicrons("layer_height_0")
-                + layer_nr * layer_thickness;
+
+            if (use_variable_layer_heights)
+            {
+                meshStorage.layers[layer_nr].printZ = adaptive_layer_heights->getLayers()->at(layer_nr).z_position;
+                meshStorage.layers[layer_nr].thickness = adaptive_layer_heights->getLayers()->at(layer_nr).layer_height;
+            }
+            else
+            {
+                meshStorage.layers[layer_nr].printZ = initial_layer_thickness + (layer_nr * layer_thickness);
+
+                if (layer_nr == 0)
+                {
+                    meshStorage.layers[layer_nr].thickness = initial_layer_thickness;
+                }
+                else
+                {
+                    meshStorage.layers[layer_nr].thickness = layer_thickness;
+                }
+            }
+
+            // add the raft offset to each layer
             if (has_raft)
             {
                 ExtruderTrain* train = storage.meshgroup->getExtruderTrain(getSettingAsIndex("adhesion_extruder_nr"));
@@ -181,6 +245,7 @@ bool FffPolygonGenerator::sliceModel(MeshGroup* meshgroup, TimeKeeper& timeKeepe
                     Raft::getTotalThickness(storage)
                     + train->getSettingInMicrons("raft_airgap")
                     - train->getSettingInMicrons("layer_0_z_overlap"); // shift all layers (except 0) down
+
                 if (layer_nr == 0)
                 {
                     layer.printZ += train->getSettingInMicrons("layer_0_z_overlap"); // undo shifting down of first layer
@@ -235,20 +300,32 @@ void FffPolygonGenerator::slices2polygons(SliceDataStorage& storage, TimeKeeper&
         Progress::messageProgress(Progress::Stage::INSET_SKIN, mesh_order_idx + 1, storage.meshes.size());
     }
 
+    if (isEmptyLayer(storage, 0) && !isEmptyLayer(storage, 1))
+    {
+        // the first layer is empty, the second is not empty, so remove the empty first layer as support isn't going to be generated under it.
+        // Do this irrespective of the value of remove_empty_first_layers as that setting is hidden when support is enabled and so cannot be relied upon
+
+        removeEmptyFirstLayers(storage, getSettingInMicrons("layer_height"), storage.print_layer_count); // changes storage.print_layer_count!
+    }
+
     log("Layer count: %i\n", storage.print_layer_count);
 
     //layerparts2HTML(storage, "output/output.html");
 
     Progress::messageProgressStage(Progress::Stage::SUPPORT, &time_keeper);
 
-    AreaSupport::generateSupportAreas(storage, storage.print_layer_count);
+    AreaSupport::generateOverhangAreas(storage);
+    AreaSupport::generateSupportAreas(storage);
+    TreeSupport tree_support_generator;
+    tree_support_generator.generateSupportAreas(storage);
 
-    // we need to remove empty layers after we have procesed the insets
+    // we need to remove empty layers after we have processed the insets
     // processInsets might throw away parts if they have no wall at all (cause it doesn't fit)
     // brim depends on the first layer not being empty
     // only remove empty layers if we haven't generate support, because then support was added underneath the model.
-    //   for some materials it's better to print on support than on the buildplate.
-    if (getSettingBoolean("remove_empty_first_layers")) {
+    //   for some materials it's better to print on support than on the build plate.
+    if (getSettingBoolean("remove_empty_first_layers"))
+    {
         removeEmptyFirstLayers(storage, getSettingInMicrons("layer_height"), storage.print_layer_count); // changes storage.print_layer_count!
     }
     if (storage.print_layer_count == 0)
@@ -439,6 +516,8 @@ void FffPolygonGenerator::processPerimeterGaps(SliceDataStorage& storage)
         constexpr int perimeter_gaps_extra_offset = 15; // extra offset so that the perimeter gaps aren't created everywhere due to rounding errors
         bool fill_perimeter_gaps = mesh.getSettingAsFillPerimeterGapMode("fill_perimeter_gaps") != FillPerimeterGapMode::NOWHERE
                                     && !getSettingBoolean("magic_spiralize");
+        bool filter_out_tiny_gaps = mesh.getSettingBoolean("filter_out_tiny_gaps");
+
         if (!fill_perimeter_gaps)
         {
             continue;
@@ -476,7 +555,9 @@ void FffPolygonGenerator::processPerimeterGaps(SliceDataStorage& storage)
                     part.perimeter_gaps.add(outer.difference(inner));
                 }
 
-                part.perimeter_gaps.removeSmallAreas(2 * INT2MM(wall_line_width_0) * INT2MM(wall_line_width_0)); // remove small outline gaps to reduce blobs on outside of model
+                if (filter_out_tiny_gaps) {
+                    part.perimeter_gaps.removeSmallAreas(2 * INT2MM(wall_line_width_0) * INT2MM(wall_line_width_0)); // remove small outline gaps to reduce blobs on outside of model
+                }
 
                 // gap between inner wall and skin/infill
                 if (fill_gaps_between_inner_wall_and_skin_or_infill && part.insets.size() > 0)
@@ -509,7 +590,9 @@ void FffPolygonGenerator::processPerimeterGaps(SliceDataStorage& storage)
                             skin_part.perimeter_gaps.add(outer.difference(inner));
                         }
 
-                        skin_part.perimeter_gaps.removeSmallAreas(2 * INT2MM(wall_line_width_0) * INT2MM(wall_line_width_0)); // remove small outline gaps to reduce blobs on outside of model
+                        if (filter_out_tiny_gaps) {
+                            skin_part.perimeter_gaps.removeSmallAreas(2 * INT2MM(wall_line_width_0) * INT2MM(wall_line_width_0)); // remove small outline gaps to reduce blobs on outside of model
+                        }
                     }
                 }
             }
@@ -658,9 +741,10 @@ void FffPolygonGenerator::processInsets(const SliceDataStorage& storage, SliceMe
         {
             inset_count += ((layer_nr % 2) + 2) % 2;
         }
-        bool recompute_outline_based_on_outer_wall = mesh.getSettingBoolean("support_enable") && !mesh.getSettingBoolean("fill_outline_gaps");
+        bool recompute_outline_based_on_outer_wall = (mesh.getSettingBoolean("support_enable") || mesh.getSettingBoolean("support_tree_enable")) && !mesh.getSettingBoolean("fill_outline_gaps");
         bool remove_parts_with_no_insets = !mesh.getSettingBoolean("fill_outline_gaps");
-        WallsComputation walls_computation(mesh.getSettingInMicrons("wall_0_inset"), line_width_0, line_width_x, inset_count, recompute_outline_based_on_outer_wall, remove_parts_with_no_insets);
+        const bool try_line_thickness = mesh.getSettingBoolean("wall_try_line_thickness");
+        WallsComputation walls_computation(mesh.getSettingInMicrons("wall_0_inset"), line_width_0, line_width_x, inset_count, recompute_outline_based_on_outer_wall, remove_parts_with_no_insets, try_line_thickness);
         walls_computation.generateInsets(layer);
     }
     else
@@ -701,10 +785,10 @@ bool FffPolygonGenerator::isEmptyLayer(SliceDataStorage& storage, const unsigned
     return true;
 }
 
-void FffPolygonGenerator::removeEmptyFirstLayers(SliceDataStorage& storage, const int layer_height, unsigned int& total_layers)
+void FffPolygonGenerator::removeEmptyFirstLayers(SliceDataStorage& storage, const int layer_height, size_t& total_layers)
 {
     int n_empty_first_layers = 0;
-    for (unsigned int layer_idx = 0; layer_idx < total_layers; layer_idx++)
+    for (size_t layer_idx = 0; layer_idx < total_layers; layer_idx++)
     {
         if (isEmptyLayer(storage, layer_idx))
         {
@@ -757,8 +841,8 @@ void FffPolygonGenerator::processSkinsAndInfill(const SliceDataStorage& storage,
 
     if (mesh.getSettingBoolean("ironing_enabled") && (!mesh.getSettingBoolean("ironing_only_highest_layer") || (unsigned int)mesh.layer_nr_max_filled_layer == layer_nr))
     {
-        TopSurface* top_surface = new TopSurface(mesh, layer_nr); //Generate the top surface to iron over.
-        mesh.layers[layer_nr].top_surface = top_surface;
+        // Generate the top surface to iron over.
+        mesh.layers[layer_nr].top_surface.setAreasFromMeshAndLayerNumber(mesh, layer_nr);
     }
 }
 
