@@ -56,12 +56,12 @@ void InsetOrderOptimizer::processHoleInsets()
 {
     const coord_t wall_line_width_0 = mesh_config.inset0_config.getLineWidth();
     const coord_t wall_line_width_x = mesh_config.insetX_config.getLineWidth();
+    const coord_t max_gap = std::max(wall_line_width_0, wall_line_width_x) * 1.1f; // if polys are closer than this, they are considered adjacent
     const coord_t wall_0_wipe_dist = mesh.getSettingInMicrons("wall_0_wipe_dist");
     const bool retract_before_outer_wall = mesh.getSettingBoolean("travel_retract_before_outer_wall");
     const bool outer_inset_first = mesh.getSettingBoolean("outer_inset_first")
         || (layer_nr == 0 && mesh.getSettingAsPlatformAdhesion("adhesion_type") == EPlatformAdhesion::BRIM && !mesh.getSettingBoolean("brim_outside_only"));
     const unsigned int num_insets = part.insets.size();
-    constexpr bool spiralize = false;
     constexpr float flow = 1.0;
 
     if (!outer_inset_first && mesh.getSettingBoolean("infill_before_walls"))
@@ -100,7 +100,7 @@ void InsetOrderOptimizer::processHoleInsets()
             {
                 gcode_writer.setExtruder_addPrime(storage, gcode_layer, extruder_nr);
                 gcode_layer.setIsInside(true); // going to print stuff inside print object
-                gcode_layer.addPolygonsByOptimizer(insets_that_do_not_surround_holes, mesh_config.insetX_config, wall_overlapper_x);
+                gcode_layer.addWalls(insets_that_do_not_surround_holes, mesh_config.insetX_config, mesh_config.bridge_insetX_config, wall_overlapper_x);
                 added_something = true;
             }
         }
@@ -127,7 +127,9 @@ void InsetOrderOptimizer::processHoleInsets()
         const unsigned outer_poly_start_idx = order_optimizer.polyStart[0];
         start_point = (*inset_polys[0][0])[outer_poly_start_idx];
     }
-    PathOrderOptimizer order_optimizer(start_point, z_seam_config);
+    Polygons comb_boundary(*gcode_layer.getCombBoundaryInside());
+    comb_boundary.simplify(100, 100);
+    PathOrderOptimizer order_optimizer(start_point, z_seam_config, &comb_boundary);
     for (unsigned int poly_idx = 1; poly_idx < inset_polys[0].size(); poly_idx++)
     {
         order_optimizer.addPolygon(*inset_polys[0][poly_idx]);
@@ -145,7 +147,6 @@ void InsetOrderOptimizer::processHoleInsets()
         Polygons hole_outer_wall; // the outermost wall of a hole
         hole_outer_wall.add(*inset_polys[0][order_optimizer.polyOrder[outer_poly_order_idx] + 1]); // +1 because first element (part outer wall) wasn't included
         std::vector<unsigned int> hole_level_1_wall_indices; // the indices of the walls that touch the hole's outer wall
-        const coord_t max_gap = std::max(wall_line_width_0, wall_line_width_x) * 1.1f; // if polys are closer than this, they are considered adjacent
         if (inset_polys.size() > 1)
         {
             // find the adjacent poly in the level 1 insets that encloses the hole
@@ -264,9 +265,9 @@ void InsetOrderOptimizer::processHoleInsets()
             {
                 if (extruder_nr == mesh.getSettingAsExtruderNr("wall_0_extruder_nr"))
                 {
-                    gcode_layer.addPolygonsByOptimizer(hole_outer_wall, mesh_config.inset0_config, wall_overlapper_0, z_seam_config, wall_0_wipe_dist, spiralize, flow, retract_before_outer_wall);
+                    gcode_layer.addWalls(hole_outer_wall, mesh_config.inset0_config, mesh_config.bridge_inset0_config, wall_overlapper_0, z_seam_config, wall_0_wipe_dist, flow, retract_before_outer_wall);
                 }
-                gcode_layer.addPolygonsByOptimizer(hole_inner_walls, mesh_config.insetX_config, wall_overlapper_x);
+                gcode_layer.addWalls(hole_inner_walls, mesh_config.insetX_config, mesh_config.bridge_insetX_config, wall_overlapper_x);
             }
             else
             {
@@ -274,16 +275,24 @@ void InsetOrderOptimizer::processHoleInsets()
                 // avoid the possible retract when moving from the end of the immediately enclosing inset to the start
                 // of the hole outer wall we first move to a location that is close to the z seam and at a vertex of the
                 // first inset we want to be printed
-                const unsigned outer_poly_start_idx = order_optimizer.polyStart[order_optimizer.polyOrder[outer_poly_order_idx]];
+                unsigned outer_poly_start_idx = order_optimizer.polyStart[order_optimizer.polyOrder[outer_poly_order_idx]];
+
+                // detect special case where where the z-seam is located on the sharpest corner and there is only 1 hole and
+                // the gap between the walls is just a few line widths
+                if (z_seam_config.type == EZSeamType::SHARPEST_CORNER && inset_polys[0].size() == 2 && PolygonUtils::polygonOutlinesAdjacent(*inset_polys[0][1], *inset_polys[0][0], max_gap * 4))
+                {
+                    // align z-seam of hole with z-seam of outer wall - makes a nicer job when printing tubes
+                    outer_poly_start_idx = PolygonUtils::findNearestVert(start_point, hole_outer_wall.back());
+                }
                 const Point z_seam_location = hole_outer_wall[0][outer_poly_start_idx];
                 // move to the location of the vertex in the outermost enclosing inset that's closest to the z seam location
                 const Point dest = hole_inner_walls.back()[PolygonUtils::findNearestVert(z_seam_location, hole_inner_walls.back())];
                 gcode_layer.addTravel(dest);
                 std::reverse(hole_inner_walls.begin(), hole_inner_walls.end());
-                gcode_layer.addPolygonsByOptimizer(hole_inner_walls, mesh_config.insetX_config, wall_overlapper_x);
+                gcode_layer.addWalls(hole_inner_walls, mesh_config.insetX_config, mesh_config.bridge_insetX_config, wall_overlapper_x);
                 if (extruder_nr == mesh.getSettingAsExtruderNr("wall_0_extruder_nr"))
                 {
-                    gcode_layer.addPolygon(hole_outer_wall[0], outer_poly_start_idx, mesh_config.inset0_config, wall_overlapper_0, wall_0_wipe_dist, spiralize, flow, retract_before_outer_wall);
+                    gcode_layer.addWall(hole_outer_wall[0], outer_poly_start_idx, mesh_config.inset0_config, mesh_config.bridge_inset0_config, wall_overlapper_0, wall_0_wipe_dist, flow, retract_before_outer_wall);
                     // move inside so an immediately following retract doesn't occur on the outer wall
                     moveInside();
                 }
@@ -295,7 +304,19 @@ void InsetOrderOptimizer::processHoleInsets()
             // just the outer wall, no level 1 insets
             gcode_writer.setExtruder_addPrime(storage, gcode_layer, extruder_nr);
             gcode_layer.setIsInside(true); // going to print stuff inside print object
-            gcode_layer.addPolygonsByOptimizer(hole_outer_wall, mesh_config.inset0_config, wall_overlapper_0, z_seam_config, wall_0_wipe_dist, spiralize, flow, retract_before_outer_wall);
+
+            // detect special case where where the z-seam is located on the sharpest corner and there is only 1 hole and
+            // the gap between the walls is just a few line widths
+            if (z_seam_config.type == EZSeamType::SHARPEST_CORNER && inset_polys[0].size() == 2 && PolygonUtils::polygonOutlinesAdjacent(*inset_polys[0][1], *inset_polys[0][0], max_gap * 2))
+            {
+                // align z-seam of hole with z-seam of outer wall - makes a nicer job when printing tubes
+                const unsigned point_idx = PolygonUtils::findNearestVert(start_point, hole_outer_wall.back());
+                gcode_layer.addWall(hole_outer_wall.back(), point_idx, mesh_config.inset0_config, mesh_config.bridge_inset0_config, wall_overlapper_0, wall_0_wipe_dist, flow, retract_before_outer_wall);
+            }
+            else
+            {
+                gcode_layer.addWalls(hole_outer_wall, mesh_config.inset0_config, mesh_config.bridge_inset0_config, wall_overlapper_0, z_seam_config, wall_0_wipe_dist, flow, retract_before_outer_wall);
+            }
             // move inside so an immediately following retract doesn't occur on the outer wall
             moveInside();
             added_something = true;
@@ -311,7 +332,6 @@ void InsetOrderOptimizer::processOuterWallInsets()
     const bool outer_inset_first = mesh.getSettingBoolean("outer_inset_first")
                                     || (layer_nr == 0 && mesh.getSettingAsPlatformAdhesion("adhesion_type") == EPlatformAdhesion::BRIM);
     const unsigned int num_insets = part.insets.size();
-    constexpr bool spiralize = false;
     constexpr float flow = 1.0;
 
     // process the part's outer wall and the level 1 insets that it surrounds
@@ -369,9 +389,9 @@ void InsetOrderOptimizer::processOuterWallInsets()
                 {
                     Polygons part_outer_wall;
                     part_outer_wall.add(*inset_polys[0][0]);
-                    gcode_layer.addPolygonsByOptimizer(part_outer_wall, mesh_config.inset0_config, wall_overlapper_0, z_seam_config, wall_0_wipe_dist, spiralize, flow, retract_before_outer_wall);
+                    gcode_layer.addWalls(part_outer_wall, mesh_config.inset0_config, mesh_config.bridge_inset0_config, wall_overlapper_0, z_seam_config, wall_0_wipe_dist, flow, retract_before_outer_wall);
                 }
-                gcode_layer.addPolygonsByOptimizer(part_inner_walls, mesh_config.insetX_config, wall_overlapper_x);
+                gcode_layer.addWalls(part_inner_walls, mesh_config.insetX_config, mesh_config.bridge_insetX_config, wall_overlapper_x);
             }
             else
             {
@@ -394,10 +414,10 @@ void InsetOrderOptimizer::processOuterWallInsets()
                     const Point dest = part_inner_walls[0][PolygonUtils::findNearestVert(z_seam_location, part_inner_walls[0])];
                     gcode_layer.addTravel(dest);
                 }
-                gcode_layer.addPolygonsByOptimizer(part_inner_walls, mesh_config.insetX_config, wall_overlapper_x);
+                gcode_layer.addWalls(part_inner_walls, mesh_config.insetX_config, mesh_config.bridge_insetX_config, wall_overlapper_x);
                 if (extruder_nr == mesh.getSettingAsExtruderNr("wall_0_extruder_nr"))
                 {
-                    gcode_layer.addPolygon(*inset_polys[0][0], outer_poly_start_idx, mesh_config.inset0_config, wall_overlapper_0, wall_0_wipe_dist, spiralize, flow, retract_before_outer_wall);
+                    gcode_layer.addWall(*inset_polys[0][0], outer_poly_start_idx, mesh_config.inset0_config, mesh_config.bridge_inset0_config, wall_overlapper_0, wall_0_wipe_dist, flow, retract_before_outer_wall);
                     // move inside so an immediately following retract doesn't occur on the outer wall
                     moveInside();
                 }
@@ -412,7 +432,7 @@ void InsetOrderOptimizer::processOuterWallInsets()
             gcode_layer.setIsInside(true); // going to print stuff inside print object
             Polygons part_outer_wall;
             part_outer_wall.add(*inset_polys[0][0]);
-            gcode_layer.addPolygonsByOptimizer(part_outer_wall, mesh_config.inset0_config, wall_overlapper_0, z_seam_config, wall_0_wipe_dist, spiralize, flow, retract_before_outer_wall);
+            gcode_layer.addWalls(part_outer_wall, mesh_config.inset0_config, mesh_config.bridge_inset0_config, wall_overlapper_0, z_seam_config, wall_0_wipe_dist, flow, retract_before_outer_wall);
             // move inside so an immediately following retract doesn't occur on the outer wall
             moveInside();
             added_something = true;
@@ -507,7 +527,7 @@ bool InsetOrderOptimizer::processInsetsWithOptimizedOrdering()
         {
             gcode_writer.setExtruder_addPrime(storage, gcode_layer, extruder_nr);
             gcode_layer.setIsInside(true); // going to print stuff inside print object
-            gcode_layer.addPolygonsByOptimizer(remaining, mesh_config.insetX_config, wall_overlapper_x);
+            gcode_layer.addWalls(remaining, mesh_config.insetX_config, mesh_config.bridge_insetX_config, wall_overlapper_x);
             added_something = true;
         }
     }
@@ -531,13 +551,12 @@ bool InsetOrderOptimizer::optimizingInsetsIsWorthwhile(const SliceMeshStorage& m
         // optimization disabled
         return false;
     }
-    const unsigned int num_insets = part.insets.size();
-    if (num_insets < 2)
+    if (part.insets.size() < 2 || part.insets[0].size() < 2)
     {
-        // only 1 inset, definitely not worth optimizing
+        // only a single outline or no holes, not worth optimizing as the original inset processing code now aligns the z-seams of the outside walls
         return false;
     }
-    // the default is to optimize as it will make the inner insets start near to the z seam location
+    // optimize all other combinations of walls and holes
     return true;
 }
 
