@@ -259,7 +259,11 @@ void slice(int argc, char **argv)
                         break;
                     case 'g':
 #pragma GCC diagnostic push
+#ifdef __GNUC__
+#if GNUC > 4
 #pragma GCC diagnostic ignored "-Wimplicit-fallthrough" //Fall-through is intended.
+#endif
+#endif
                         last_settings_object = meshgroup;
 #pragma GCC diagnostic pop
                     case 's':
@@ -346,7 +350,7 @@ int main(int argc, char **argv)
     std::cerr << std::boolalpha;
     logAlways("\n");
     logAlways("Cura_SteamEngine version %s\n", VERSION);
-    logAlways("Copyright (C) 2017 Ultimaker\n");
+    logAlways("Copyright (C) 2018 Ultimaker\n");
     logAlways("\n");
     logAlways("This program is free software: you can redistribute it and/or modify\n");
     logAlways("it under the terms of the GNU Affero General Public License as published by\n");
