@@ -3,7 +3,7 @@
 
 #include "settings.h" // MAX_INFILL_COMBINE
 #include "../sliceDataStorage.h" // SliceDataStorage
-
+#include "../raft.h"
 
 namespace cura
 {
@@ -27,7 +27,7 @@ std::vector<double> PathConfigStorage::getLineWidthFactorPerExtruder(const Slice
     return ret;
 }
 
-GCodePathConfig createPerimeterGapConfig(const SliceMeshStorage& mesh, int layer_thickness)
+GCodePathConfig createPerimeterGapConfig(const SliceMeshStorage& mesh, int layer_thickness, int layer_nr)
 {
     // The perimeter gap config follows the skin config, but has a different line width:
     // wall_line_width_x divided by two because the gaps are between 0 and 1 times the wall line width
@@ -42,49 +42,94 @@ GCodePathConfig createPerimeterGapConfig(const SliceMeshStorage& mesh, int layer
             PrintFeatureType::Skin
             , perimeter_gaps_line_width
             , layer_thickness
-            , mesh.getSettingInPercentage("material_flow")
+            , (layer_nr == 0)? mesh.getSettingInPercentage("material_flow_layer_0") : mesh.getSettingInPercentage("material_flow")
             , GCodePathConfig::SpeedDerivatives{perimeter_gaps_speed, mesh.getSettingInMillimetersPerSecond("acceleration_topbottom"), mesh.getSettingInMillimetersPerSecond("jerk_topbottom")}
         );
 }
 
-PathConfigStorage::MeshPathConfigs::MeshPathConfigs(const SliceMeshStorage& mesh, int layer_thickness, const std::vector<double>& line_width_factor_per_extruder)
+PathConfigStorage::MeshPathConfigs::MeshPathConfigs(const SliceMeshStorage& mesh, int layer_thickness, int layer_nr, const std::vector<double>& line_width_factor_per_extruder)
 : inset0_config(
     PrintFeatureType::OuterWall
     , mesh.getSettingInMicrons("wall_line_width_0") * line_width_factor_per_extruder[mesh.getSettingAsExtruderNr("wall_0_extruder_nr")]
     , layer_thickness
-    , mesh.getSettingInPercentage("material_flow")
+    , (layer_nr == 0)? mesh.getSettingInPercentage("material_flow_layer_0") : mesh.getSettingInPercentage("material_flow")
     , GCodePathConfig::SpeedDerivatives{mesh.getSettingInMillimetersPerSecond("speed_wall_0"), mesh.getSettingInMillimetersPerSecond("acceleration_wall_0"), mesh.getSettingInMillimetersPerSecond("jerk_wall_0")}
 )
 , insetX_config(
     PrintFeatureType::InnerWall
     , mesh.getSettingInMicrons("wall_line_width_x") * line_width_factor_per_extruder[mesh.getSettingAsExtruderNr("wall_x_extruder_nr")]
     , layer_thickness
-    , mesh.getSettingInPercentage("material_flow")
+    , (layer_nr == 0)? mesh.getSettingInPercentage("material_flow_layer_0") : mesh.getSettingInPercentage("material_flow")
     , GCodePathConfig::SpeedDerivatives{mesh.getSettingInMillimetersPerSecond("speed_wall_x"), mesh.getSettingInMillimetersPerSecond("acceleration_wall_x"), mesh.getSettingInMillimetersPerSecond("jerk_wall_x")}
 )
+, bridge_inset0_config(
+    PrintFeatureType::OuterWall
+    , mesh.getSettingInMicrons("wall_line_width_0") * line_width_factor_per_extruder[mesh.getSettingAsExtruderNr("wall_0_extruder_nr")]
+    , layer_thickness
+    , mesh.getSettingInPercentage("bridge_wall_material_flow")
+    , GCodePathConfig::SpeedDerivatives{mesh.getSettingInMillimetersPerSecond("bridge_wall_speed"), mesh.getSettingInMillimetersPerSecond("acceleration_wall_0"), mesh.getSettingInMillimetersPerSecond("jerk_wall_0")}
+    , true // is_bridge_path
+    , mesh.getSettingInPercentage("bridge_fan_speed")
+)
+, bridge_insetX_config(
+    PrintFeatureType::InnerWall
+    , mesh.getSettingInMicrons("wall_line_width_x") * line_width_factor_per_extruder[mesh.getSettingAsExtruderNr("wall_x_extruder_nr")]
+    , layer_thickness
+    , mesh.getSettingInPercentage("bridge_wall_material_flow")
+    , GCodePathConfig::SpeedDerivatives{mesh.getSettingInMillimetersPerSecond("bridge_wall_speed"), mesh.getSettingInMillimetersPerSecond("acceleration_wall_x"), mesh.getSettingInMillimetersPerSecond("jerk_wall_x")}
+    , true // is_bridge_path
+    , mesh.getSettingInPercentage("bridge_fan_speed")
+)
 , skin_config(
     PrintFeatureType::Skin
     , mesh.getSettingInMicrons("skin_line_width") * line_width_factor_per_extruder[mesh.getSettingAsExtruderNr("top_bottom_extruder_nr")]
     , layer_thickness
-    , mesh.getSettingInPercentage("material_flow")
+    , (layer_nr == 0)? mesh.getSettingInPercentage("material_flow_layer_0") : mesh.getSettingInPercentage("material_flow")
     , GCodePathConfig::SpeedDerivatives{mesh.getSettingInMillimetersPerSecond("speed_topbottom"), mesh.getSettingInMillimetersPerSecond("acceleration_topbottom"), mesh.getSettingInMillimetersPerSecond("jerk_topbottom")}
 )
+, bridge_skin_config( // use bridge skin flow, speed and fan
+    PrintFeatureType::Skin
+    , mesh.getSettingInMicrons("skin_line_width") * line_width_factor_per_extruder[mesh.getSettingAsExtruderNr("top_bottom_extruder_nr")]
+    , layer_thickness
+    , mesh.getSettingInPercentage("bridge_skin_material_flow")
+    , GCodePathConfig::SpeedDerivatives{mesh.getSettingInMillimetersPerSecond("bridge_skin_speed"), mesh.getSettingInMillimetersPerSecond("acceleration_topbottom"), mesh.getSettingInMillimetersPerSecond("jerk_topbottom")}
+    , true // is_bridge_path
+    , mesh.getSettingInPercentage("bridge_fan_speed")
+)
+, bridge_skin_config2( // use bridge skin 2 flow, speed and fan
+    PrintFeatureType::Skin
+    , mesh.getSettingInMicrons("skin_line_width") * line_width_factor_per_extruder[mesh.getSettingAsExtruderNr("top_bottom_extruder_nr")]
+    , layer_thickness
+    , mesh.getSettingInPercentage("bridge_skin_material_flow_2")
+    , GCodePathConfig::SpeedDerivatives{mesh.getSettingInMillimetersPerSecond("bridge_skin_speed_2"), mesh.getSettingInMillimetersPerSecond("acceleration_topbottom"), mesh.getSettingInMillimetersPerSecond("jerk_topbottom")}
+    , true // is_bridge_path
+    , mesh.getSettingInPercentage("bridge_fan_speed_2")
+)
+, bridge_skin_config3( // use bridge skin 3 flow, speed and fan
+    PrintFeatureType::Skin
+    , mesh.getSettingInMicrons("skin_line_width") * line_width_factor_per_extruder[mesh.getSettingAsExtruderNr("top_bottom_extruder_nr")]
+    , layer_thickness
+    , mesh.getSettingInPercentage("bridge_skin_material_flow_3")
+    , GCodePathConfig::SpeedDerivatives{mesh.getSettingInMillimetersPerSecond("bridge_skin_speed_3"), mesh.getSettingInMillimetersPerSecond("acceleration_topbottom"), mesh.getSettingInMillimetersPerSecond("jerk_topbottom")}
+    , true // is_bridge_path
+    , mesh.getSettingInPercentage("bridge_fan_speed_3")
+)
 , roofing_config(
     PrintFeatureType::Skin
     , mesh.getSettingInMicrons("roofing_line_width")
     , layer_thickness
-    , mesh.getSettingInPercentage("material_flow")
+    , (layer_nr == 0)? mesh.getSettingInPercentage("material_flow_layer_0") : mesh.getSettingInPercentage("material_flow")
     , GCodePathConfig::SpeedDerivatives{mesh.getSettingInMillimetersPerSecond("speed_roofing"), mesh.getSettingInMillimetersPerSecond("acceleration_roofing"), mesh.getSettingInMillimetersPerSecond("jerk_roofing")}
 )
 , ironing_config(
     PrintFeatureType::Skin
     , mesh.getSettingInMicrons("skin_line_width")
     , layer_thickness
-    , mesh.getSettingInPercentage("material_flow")
+    , (layer_nr == 0)? mesh.getSettingInPercentage("material_flow_layer_0") : mesh.getSettingInPercentage("material_flow")
     , GCodePathConfig::SpeedDerivatives{mesh.getSettingInMillimetersPerSecond("speed_ironing"), mesh.getSettingInMillimetersPerSecond("acceleration_ironing"), mesh.getSettingInMillimetersPerSecond("jerk_ironing")}
 )
 
-, perimeter_gap_config(createPerimeterGapConfig(mesh, layer_thickness))
+, perimeter_gap_config(createPerimeterGapConfig(mesh, layer_thickness, layer_nr))
 {
     infill_config.reserve(MAX_INFILL_COMBINE);
     for (int combine_idx = 0; combine_idx < MAX_INFILL_COMBINE; combine_idx++)
@@ -93,7 +138,7 @@ PathConfigStorage::MeshPathConfigs::MeshPathConfigs(const SliceMeshStorage& mesh
                 PrintFeatureType::Infill
                 , mesh.getSettingInMicrons("infill_line_width") * (combine_idx + 1) * line_width_factor_per_extruder[mesh.getSettingAsExtruderNr("infill_extruder_nr")]
                 , layer_thickness
-                , mesh.getSettingInPercentage("material_flow")
+                , (layer_nr == 0)? mesh.getSettingInPercentage("material_flow_layer_0") : mesh.getSettingInPercentage("material_flow")
                 , GCodePathConfig::SpeedDerivatives{mesh.getSettingInMillimetersPerSecond("speed_infill"), mesh.getSettingInMillimetersPerSecond("acceleration_infill"), mesh.getSettingInMillimetersPerSecond("jerk_infill")}
             );
     }
@@ -113,35 +158,35 @@ PathConfigStorage::PathConfigStorage(const SliceDataStorage& storage, int layer_
             PrintFeatureType::SupportInterface
             , adhesion_extruder_train->getSettingInMicrons("raft_base_line_width")
             , adhesion_extruder_train->getSettingInMicrons("raft_base_thickness")
-            , adhesion_extruder_train->getSettingInPercentage("material_flow")
+            , (layer_nr == 0)? adhesion_extruder_train->getSettingInPercentage("material_flow_layer_0") : adhesion_extruder_train->getSettingInPercentage("material_flow")
             , GCodePathConfig::SpeedDerivatives{adhesion_extruder_train->getSettingInMillimetersPerSecond("raft_base_speed"), adhesion_extruder_train->getSettingInMillimetersPerSecond("raft_base_acceleration"), adhesion_extruder_train->getSettingInMillimetersPerSecond("raft_base_jerk")}
         )
 , raft_interface_config(
             PrintFeatureType::Support
             , adhesion_extruder_train->getSettingInMicrons("raft_interface_line_width")
             , adhesion_extruder_train->getSettingInMicrons("raft_interface_thickness")
-            , adhesion_extruder_train->getSettingInPercentage("material_flow")
+            , (layer_nr == 0)? adhesion_extruder_train->getSettingInPercentage("material_flow_layer_0") : adhesion_extruder_train->getSettingInPercentage("material_flow")
             , GCodePathConfig::SpeedDerivatives{adhesion_extruder_train->getSettingInMillimetersPerSecond("raft_interface_speed"), adhesion_extruder_train->getSettingInMillimetersPerSecond("raft_interface_acceleration"), adhesion_extruder_train->getSettingInMillimetersPerSecond("raft_interface_jerk")}
         )
 , raft_surface_config(
             PrintFeatureType::SupportInterface
             , adhesion_extruder_train->getSettingInMicrons("raft_surface_line_width")
             , adhesion_extruder_train->getSettingInMicrons("raft_surface_thickness")
-            , adhesion_extruder_train->getSettingInPercentage("material_flow")
+            , (layer_nr == 0)? adhesion_extruder_train->getSettingInPercentage("material_flow_layer_0") : adhesion_extruder_train->getSettingInPercentage("material_flow")
             , GCodePathConfig::SpeedDerivatives{adhesion_extruder_train->getSettingInMillimetersPerSecond("raft_surface_speed"), adhesion_extruder_train->getSettingInMillimetersPerSecond("raft_surface_acceleration"), adhesion_extruder_train->getSettingInMillimetersPerSecond("raft_surface_jerk")}
         )
 , support_roof_config(
             PrintFeatureType::SupportInterface
             , support_roof_train->getSettingInMicrons("support_roof_line_width") * line_width_factor_per_extruder[support_roof_extruder_nr]
             , layer_thickness
-            , support_roof_train->getSettingInPercentage("material_flow")
+            , (layer_nr == 0)? support_roof_train->getSettingInPercentage("material_flow_layer_0") : support_roof_train->getSettingInPercentage("material_flow")
             , GCodePathConfig::SpeedDerivatives{support_roof_train->getSettingInMillimetersPerSecond("speed_support_roof"), support_roof_train->getSettingInMillimetersPerSecond("acceleration_support_roof"), support_roof_train->getSettingInMillimetersPerSecond("jerk_support_roof")}
         )
 , support_bottom_config(
             PrintFeatureType::SupportInterface
             , support_bottom_train->getSettingInMicrons("support_bottom_line_width") * line_width_factor_per_extruder[support_bottom_extruder_nr]
             , layer_thickness
-            , support_bottom_train->getSettingInPercentage("material_flow")
+            , (layer_nr == 0)? support_bottom_train->getSettingInPercentage("material_flow_layer_0") : support_bottom_train->getSettingInPercentage("material_flow")
             , GCodePathConfig::SpeedDerivatives{support_bottom_train->getSettingInMillimetersPerSecond("speed_support_bottom"), support_bottom_train->getSettingInMillimetersPerSecond("acceleration_support_bottom"), support_bottom_train->getSettingInMillimetersPerSecond("jerk_support_bottom")}
         )
 {
@@ -164,7 +209,7 @@ PathConfigStorage::PathConfigStorage(const SliceDataStorage& storage, int layer_
                 , train->getSettingInMicrons("skirt_brim_line_width")
                     * ((storage.getSettingAsPlatformAdhesion("adhesion_type") == EPlatformAdhesion::RAFT) ? 1.0 : line_width_factor_per_extruder[extruder_nr]) // cause it's also used for the draft/ooze shield
                 , layer_thickness
-                , train->getSettingInPercentage("material_flow")
+                , (layer_nr == 0)? train->getSettingInPercentage("material_flow_layer_0") : train->getSettingInPercentage("material_flow")
                 , GCodePathConfig::SpeedDerivatives{train->getSettingInMillimetersPerSecond("skirt_brim_speed"), train->getSettingInMillimetersPerSecond("acceleration_skirt_brim"), train->getSettingInMillimetersPerSecond("jerk_skirt_brim")}
             );
         prime_tower_config_per_extruder.emplace_back(
@@ -180,7 +225,7 @@ PathConfigStorage::PathConfigStorage(const SliceDataStorage& storage, int layer_
     mesh_configs.reserve(storage.meshes.size());
     for (const SliceMeshStorage& mesh_storage : storage.meshes)
     {
-        mesh_configs.emplace_back(mesh_storage, layer_thickness, line_width_factor_per_extruder);
+        mesh_configs.emplace_back(mesh_storage, layer_thickness, layer_nr, line_width_factor_per_extruder);
     }
 
     support_infill_config.reserve(MAX_INFILL_COMBINE);
@@ -191,13 +236,13 @@ PathConfigStorage::PathConfigStorage(const SliceDataStorage& storage, int layer_
             PrintFeatureType::Support
             , support_infill_train->getSettingInMicrons("support_line_width") * (combine_idx + 1) * support_infill_line_width_factor
             , layer_thickness
-            , support_infill_train->getSettingInPercentage("material_flow")
+            , (layer_nr == 0)? support_infill_train->getSettingInPercentage("material_flow_layer_0") : support_infill_train->getSettingInPercentage("material_flow")
             , GCodePathConfig::SpeedDerivatives{support_infill_train->getSettingInMillimetersPerSecond("speed_support_infill"), support_infill_train->getSettingInMillimetersPerSecond("acceleration_support_infill"), support_infill_train->getSettingInMillimetersPerSecond("jerk_support_infill")}
         );
     }
 
     const int initial_speedup_layer_count = storage.getSettingAsCount("speed_slowdown_layers");
-    if (layer_nr < initial_speedup_layer_count)
+    if (layer_nr >= 0 && layer_nr < initial_speedup_layer_count)
     {
         handleInitialLayerSpeedup(storage, layer_nr, initial_speedup_layer_count);
     }
