@@ -7,7 +7,7 @@
 #include <vector>
 #include <assert.h>
 #include <float.h>
-#include <clipper/clipper.hpp>
+#include <clipper.hpp>
 
 #include <algorithm>    // std::reverse, fill_n array
 #include <cmath> // fabs
@@ -69,10 +69,18 @@ public:
 
     ConstPolygonRef& operator=(const ConstPolygonRef& other) =delete; // Cannot assign to a const object
 
-    unsigned int size() const
-    {
-        return path->size();
-    }
+    /*!
+     * Gets the number of vertices in this polygon.
+     * \return The number of vertices in this polygon.
+     */
+    size_t size() const;
+
+    /*!
+     * Returns whether there are any vertices in this polygon.
+     * \return ``true`` if the polygon has no vertices at all, or ``false`` if
+     * it does have vertices.
+     */
+    bool empty() const;
 
     const Point& operator[] (unsigned int index) const
     {
@@ -1122,20 +1130,14 @@ public:
     {
         return this->paths[0];
     }
-    
-    bool inside(Point p) const
-    {
-        if (size() < 1)
-            return false;
-        if (!(*this)[0].inside(p))
-            return false;
-        for(unsigned int n=1; n<paths.size(); n++)
-        {
-            if ((*this)[n].inside(p))
-                return false;
-        }
-        return true;
-    }
+
+    /*!
+     * Tests whether the given point is inside this polygon part.
+     * \param p The point to test whether it is inside.
+     * \param border_result If the point is exactly on the border, this will be
+     * returned instead.
+     */
+    bool inside(Point p, bool border_result = false) const;
 };
 
 /*!
